package java.security;

import sun.security.util.SecurityConstants;
import java.util.ArrayList;
import sun.security.util.Debug;

public final class AccessControlContext
{
    private ProtectionDomain[] context;
    private boolean isPrivileged;
    private boolean isAuthorized;
    private AccessControlContext privilegedContext;
    private DomainCombiner combiner;
    private Permission[] permissions;
    private AccessControlContext parent;
    private boolean isWrapped;
    private boolean isLimited;
    private ProtectionDomain[] limitedContext;
    private static boolean debugInit;
    private static Debug debug;
    
    static Debug getDebug() {
        if (AccessControlContext.debugInit) {
            return AccessControlContext.debug;
        }
        if (Policy.isSet()) {
            AccessControlContext.debug = Debug.getInstance("access");
            AccessControlContext.debugInit = true;
        }
        return AccessControlContext.debug;
    }
    
    public AccessControlContext(ProtectionDomain[] array) {
        this.isAuthorized = false;
        this.combiner = null;
        if (array.length == 0) {
            this.context = null;
        }
        else if (array.length == 1) {
            if (array[0] != null) {
                this.context = (ProtectionDomain[])array.clone();
            }
            else {
                this.context = null;
            }
        }
        else {
            ArrayList list = new ArrayList(array.length);
            for (int i = 0; i < array.length; ++i) {
                if (array[i] != null && !list.contains(array[i])) {
                    list.add(array[i]);
                }
            }
            if (!list.isEmpty()) {
                this.context = new ProtectionDomain[list.size()];
                this.context = (ProtectionDomain[])list.toArray(this.context);
            }
        }
    }
    
    public AccessControlContext(AccessControlContext accessControlContext, DomainCombiner domainCombiner) {
        this(accessControlContext, domainCombiner, false);
    }
    
    AccessControlContext(AccessControlContext accessControlContext, DomainCombiner combiner, boolean b) {
        this.isAuthorized = false;
        this.combiner = null;
        if (!b) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(SecurityConstants.CREATE_ACC_PERMISSION);
                this.isAuthorized = true;
            }
        }
        else {
            this.isAuthorized = true;
        }
        this.context = accessControlContext.context;
        this.combiner = combiner;
    }
    
    AccessControlContext(ProtectionDomain protectionDomain, DomainCombiner combiner, AccessControlContext parent, AccessControlContext privilegedContext, Permission[] array) {
        this.isAuthorized = false;
        this.combiner = null;
        ProtectionDomain[] array2 = null;
        if (protectionDomain != null) {
            array2 = new ProtectionDomain[] { protectionDomain };
        }
        if (privilegedContext != null) {
            if (combiner != null) {
                this.context = combiner.combine(array2, privilegedContext.context);
            }
            else {
                this.context = combine(array2, privilegedContext.context);
            }
        }
        else if (combiner != null) {
            this.context = combiner.combine(array2, (ProtectionDomain[])null);
        }
        else {
            this.context = combine(array2, (ProtectionDomain[])null);
        }
        this.combiner = combiner;
        Permission[] permissions = null;
        if (array != null) {
            permissions = new Permission[array.length];
            int i = 0;
            while (i < array.length) {
                if (array[i] == null) {
                    throw new NullPointerException("permission can't be null");
                }
                if (array[i].getClass() == AllPermission.class) {
                    parent = null;
                }
                permissions[i] = array[i];
                ++i;
            }
        }
        if (parent != null) {
            this.limitedContext = combine(parent.context, parent.limitedContext);
            this.isLimited = true;
            this.isWrapped = true;
            this.permissions = permissions;
            this.parent = parent;
            this.privilegedContext = privilegedContext;
        }
        this.isAuthorized = true;
    }
    
    AccessControlContext(ProtectionDomain[] context, boolean isPrivileged) {
        this.isAuthorized = false;
        this.combiner = null;
        this.context = context;
        this.isPrivileged = isPrivileged;
        this.isAuthorized = true;
    }
    
    AccessControlContext(ProtectionDomain[] context, AccessControlContext privilegedContext) {
        this.isAuthorized = false;
        this.combiner = null;
        this.context = context;
        this.privilegedContext = privilegedContext;
        this.isPrivileged = true;
    }
    
    ProtectionDomain[] getContext() {
        return this.context;
    }
    
    boolean isPrivileged() {
        return this.isPrivileged;
    }
    
    DomainCombiner getAssignedCombiner() {
        AccessControlContext accessControlContext;
        if (this.isPrivileged) {
            accessControlContext = this.privilegedContext;
        }
        else {
            accessControlContext = AccessController.getInheritedAccessControlContext();
        }
        if (accessControlContext != null) {
            return accessControlContext.combiner;
        }
        return null;
    }
    
    public DomainCombiner getDomainCombiner() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.GET_COMBINER_PERMISSION);
        }
        return this.getCombiner();
    }
    
    DomainCombiner getCombiner() {
        return this.combiner;
    }
    
    boolean isAuthorized() {
        return this.isAuthorized;
    }
    
    public void checkPermission(Permission permission) throws AccessControlException {
        boolean n = false;
        if (permission == null) {
            throw new NullPointerException("permission can't be null");
        }
        if (getDebug() != null) {
            int n2 = Debug.isOn("codebase=") ? false : true;
            if (n2 == 0) {
                int n3 = 0;
                while (this.context != null && n3 < this.context.length) {
                    if (this.context[n3].getCodeSource() != null && this.context[n3].getCodeSource().getLocation() != null && Debug.isOn("codebase=" + this.context[n3].getCodeSource().getLocation().toString())) {
                        n2 = true;
                        break;
                    }
                    ++n3;
                }
            }
            n = (n2 & ((!Debug.isOn("permission=") || Debug.isOn("permission=" + permission.getClass().getCanonicalName())) ? 1 : 0));
            if (n && Debug.isOn("stack")) {
                Thread.dumpStack();
            }
            if (n && Debug.isOn("domain")) {
                if (this.context == null) {
                    AccessControlContext.debug.println("domain (context is null)");
                }
                else {
                    for (int i = 0; i < this.context.length; ++i) {
                        AccessControlContext.debug.println("domain " + i + " " + this.context[i]);
                    }
                }
            }
        }
        if (this.context == null) {
            this.checkPermission2(permission);
            return;
        }
        int j = 0;
        while (j < this.context.length) {
            if (this.context[j] != null && !this.context[j].implies(permission)) {
                if (n != 0) {
                    AccessControlContext.debug.println("access denied " + permission);
                }
                if (Debug.isOn("failure") && AccessControlContext.debug != null) {
                    if (n == 0) {
                        AccessControlContext.debug.println("access denied " + permission);
                    }
                    Thread.dumpStack();
                    AccessController.doPrivileged(new PrivilegedAction<Void>() {
                        
                        @Override
                        public Void run() {
                            db.println("domain that failed " + pd);
                            return null;
                        }
                    });
                }
                throw new AccessControlException("access denied " + permission, permission);
            }
            else {
                ++j;
            }
        }
        if (n != 0) {
            AccessControlContext.debug.println("access allowed " + permission);
        }
        this.checkPermission2(permission);
        return;
    }
    
    private void checkPermission2(Permission permission) {
        if (!this.isLimited) {
            return;
        }
        if (this.privilegedContext != null) {
            this.privilegedContext.checkPermission2(permission);
        }
        if (this.isWrapped) {
            return;
        }
        if (this.permissions != null) {
            Class<Object> class1 = permission.getClass();
            int i = 0;
            while (i < this.permissions.length) {
                Permission permission2 = this.permissions[i];
                if (permission2.getClass().equals(class1) && permission2.implies(permission)) {
                    return;
                }
                ++i;
            }
        }
        if (this.parent != null) {
            if (this.permissions == null) {
                this.parent.checkPermission2(permission);
            }
            else {
                this.parent.checkPermission(permission);
            }
        }
        return;
    }
    
    AccessControlContext optimize() {
        DomainCombiner combiner = null;
        AccessControlContext parent = null;
        Permission[] permissions = null;
        AccessControlContext accessControlContext;
        if (this.isPrivileged) {
            accessControlContext = this.privilegedContext;
            if (accessControlContext != null && accessControlContext.isWrapped) {
                permissions = accessControlContext.permissions;
                parent = accessControlContext.parent;
            }
        }
        else {
            accessControlContext = AccessController.getInheritedAccessControlContext();
            if (accessControlContext != null && accessControlContext.isLimited) {
                parent = accessControlContext;
            }
        }
        int n = this.context == null;
        int n2 = accessControlContext == null || accessControlContext.context == null;
        ProtectionDomain[] array = n2 ? null : accessControlContext.context;
        int n3 = (accessControlContext == null || !accessControlContext.isWrapped) && parent == null;
        ProtectionDomain[] context;
        if (accessControlContext != null && accessControlContext.combiner != null) {
            if (getDebug() != null) {
                AccessControlContext.debug.println("AccessControlContext invoking the Combiner");
            }
            combiner = accessControlContext.combiner;
            context = combiner.combine(this.context, array);
        }
        else {
            if (!(n == 0)) {
                if (!(n2 == 0)) {
                    this.calculateFields(accessControlContext, parent, permissions);
                    return this;
                }
                if (!(n3 == 0)) {
                    return accessControlContext;
                }
            }
            else if (array != null && n3 && this.context.length == 1 && this.context[0] == array[0]) {
                return accessControlContext;
            }
            context = combine(this.context, array);
            if (n3 && n2 == 0 && context == array) {
                return accessControlContext;
            }
            if (n2 && context == this.context) {
                this.calculateFields(accessControlContext, parent, permissions);
                return this;
            }
        }
        this.context = context;
        this.combiner = combiner;
        this.isPrivileged = false;
        this.calculateFields(accessControlContext, parent, permissions);
        return this;
    }
    
    private static ProtectionDomain[] combine(ProtectionDomain[] array, ProtectionDomain[] array2) {
        int n = array == null;
        int n2 = array2 == null;
        int n3 = n ? 0 : array.length;
        if (n2 && n3 <= 2) {
            return array;
        }
        int n4 = n2 ? 0 : array2.length;
        ProtectionDomain[] array3 = new ProtectionDomain[n3 + n4];
        if (n2 == 0) {
            System.arraycopy(array2, 0, array3, 0, n4);
        }
    Label_0140:
        for (ProtectionDomain protectionDomain : array) {
            if (protectionDomain != null) {
                int j = 0;
                while (j < n4) {
                    if (protectionDomain == array3[j]) {
                        continue Label_0140;
                    }
                    ++j;
                }
                array3[n4++] = protectionDomain;
            }
        }
        if (n4 != array3.length) {
            if (n2 == 0 && n4 == array2.length) {
                return array2;
            }
            if (n2 && n4 == n3) {
                return array;
            }
            ProtectionDomain[] array4 = new ProtectionDomain[n4];
            System.arraycopy(array3, 0, array4, 0, n4);
            array3 = array4;
        }
        return array3;
    }
    
    private void calculateFields(AccessControlContext accessControlContext, AccessControlContext parent, Permission[] permissions) {
        ProtectionDomain[] combine = combine((parent != null) ? parent.limitedContext : null, (accessControlContext != null) ? accessControlContext.limitedContext : null);
        if (combine != null && (this.context == null || !containsAllPDs(combine, this.context))) {
            this.limitedContext = combine;
            this.permissions = permissions;
            this.parent = parent;
            this.isLimited = true;
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof AccessControlContext)) {
            return false;
        }
        AccessControlContext accessControlContext = (AccessControlContext)o;
        return this.equalContext(accessControlContext) && this.equalLimitedContext(accessControlContext);
    }
    
    private boolean equalContext(AccessControlContext accessControlContext) {
        return this.equalPDs(this.context, accessControlContext.context) && (this.combiner != null || accessControlContext.combiner == null) && (this.combiner == null || this.combiner.equals(accessControlContext.combiner));
    }
    
    private boolean equalPDs(ProtectionDomain[] array, ProtectionDomain[] array2) {
        if (array == null) {
            return array2 == null;
        }
        return array2 != null && containsAllPDs(array, array2) && containsAllPDs(array2, array);
    }
    
    private boolean equalLimitedContext(AccessControlContext accessControlContext) {
        if (accessControlContext == null) {
            return false;
        }
        if (!this.isLimited && !accessControlContext.isLimited) {
            return true;
        }
        if (!this.isLimited || !accessControlContext.isLimited) {
            return false;
        }
        if ((this.isWrapped && !accessControlContext.isWrapped) || (!this.isWrapped && accessControlContext.isWrapped)) {
            return false;
        }
        if (this.permissions == null && accessControlContext.permissions != null) {
            return false;
        }
        if (this.permissions != null && accessControlContext.permissions == null) {
            return false;
        }
        if (!this.containsAllLimits(accessControlContext) || !accessControlContext.containsAllLimits(this)) {
            return false;
        }
        AccessControlContext nextPC = getNextPC(this);
        AccessControlContext nextPC2 = getNextPC(accessControlContext);
        return (nextPC != null || nextPC2 == null || !nextPC2.isLimited) && (nextPC == null || nextPC.equalLimitedContext(nextPC2)) && (this.parent != null || accessControlContext.parent == null) && (this.parent == null || this.parent.equals(accessControlContext.parent));
    }
    
    private static AccessControlContext getNextPC(AccessControlContext privilegedContext) {
        while (privilegedContext != null && privilegedContext.privilegedContext != null) {
            privilegedContext = privilegedContext.privilegedContext;
            if (privilegedContext.isWrapped) {
                continue;
            }
            return privilegedContext;
        }
        return null;
    }
    
    private static boolean containsAllPDs(ProtectionDomain[] array, ProtectionDomain[] array2) {
        boolean n = false;
        int i = 0;
        while (i < array.length) {
            n = false;
            ProtectionDomain protectionDomain;
            if ((protectionDomain = array[i]) == null) {
                for (int n3 = 0; n3 < array2.length && n == 0; n = (array2[n3] == null), ++n3) {}
            }
            else {
                Class<Object> class1 = protectionDomain.getClass();
                ProtectionDomain obj;
                for (int n2 = 0; n2 < array2.length && n == 0; n = (obj != null && class1 == obj.getClass() && protectionDomain.equals(obj)), ++n2) {
                    obj = array2[n2];
                }
            }
            if (n == 0) {
                return false;
            }
            ++i;
        }
        return n;
    }
    
    private boolean containsAllLimits(AccessControlContext accessControlContext) {
        boolean n = false;
        if (this.permissions == null && accessControlContext.permissions == null) {
            return true;
        }
        int i = 0;
        while (i < this.permissions.length) {
            Permission permission = this.permissions[i];
            Class<Object> class1 = permission.getClass();
            n = false;
            Permission permission2;
            for (int n2 = 0; n2 < accessControlContext.permissions.length && n == 0; n = (class1.equals(permission2.getClass()) && permission.equals(permission2)), ++n2) {
                permission2 = accessControlContext.permissions[n2];
            }
            if (n == 0) {
                return false;
            }
            ++i;
        }
        return n;
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        if (this.context == null) {
            return n;
        }
        for (int i = 0; i < this.context.length; ++i) {
            if (this.context[i] != null) {
                n ^= this.context[i].hashCode();
            }
        }
        return n;
    }
    
    static {
        AccessControlContext.debugInit = false;
        AccessControlContext.debug = null;
    }
}
